==> cloud-3.2.0.tar.gz: cloud/modules/cloud_service_providers/aws_cloud/src/Form/Ec2/VolumeAttachForm.php <==
<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\Core\Form\FormStateInterface;
use Drupal\cloud\Traits\CloudContentEntityTrait;

/**
 * Volume Attach form.
 */
class VolumeAttachForm extends AwsDeleteForm {

  use CloudContentEntityTrait;

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    $entity = $this->entity;

    return $this->t('Are you sure you want to attach volume: %name?', [
      '%name' => $entity->label(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Attach');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    $entity = $this->entity;
    return $this->t('<h2>Volume Information:</h2><ul><li>Volume id: %id</li><li>Volume name: %name</li></ul>', [
      '%id' => $entity->getVolumeId(),
      '%name' => $entity->getName(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {
    $form = parent::buildForm($form, $form_state);

    $entity = $this->entity;
    if (!$this->updateEntityBuildForm($entity)) {
      return $this->redirectUrl;
    }

    // Get module name.
    $module_name = $this->getModuleName($entity);

    $instances = [];
    $results = $this->getInstances($this->entity->getAvailabilityZone(), $module_name, $cloud_context);

    foreach ($results ?: [] as $result) {
      /** @var \Drupal\aws_cloud\Entity\Ec2\Instance $result */
      $instances[$result->getInstanceId()] = $this->t('%name - %instance_id', [
        '%name' => $result->getName(),
        '%instance_id' => $result->getInstanceId(),
      ]);
    }
    if (count($results) > 0) {
      $form['device_name'] = [
        '#title' => $this->t('Device Name'),
        '#type' => 'textfield',
        '#description' => $this->t('The device name (for example, /dev/sdh or xvdh).'),
        '#required' => TRUE,
      ];

      $form['instance_id'] = [
        '#type' => 'select',
        '#title' => $this->t('Instance ID'),
        '#options' => $instances,
      ];
    }
    else {
      $form['message'] = [
        '#markup' => '<h1>' . $this->t('No instances available in the availability zone: %zone.  Volume cannot be attached.', ['%zone' => $this->entity->getAvailabilityZone()]) . '</h1>',
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);

    $entity = $this->entity;

    // Get module name.
    $module_name = $this->getModuleName($entity);

    $results = $this->getInstances($this->entity->getAvailabilityZone(), $module_name, $entity->getCloudContext());
    if (count($results) === 0) {
      unset($actions['submit']);
    }
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\aws_cloud\Entity\Ec2\Volume $entity */
    $entity = $this->entity;

    $instance_id = $form_state->getValue('instance_id');
    $volume_id = $entity->getVolumeId();
    $device_name = $form_state->getValue('device_name');

    $this->ec2Service->setCloudContext($this->entity->getCloudContext());
    $result = $this->ec2Service->attachVolume([
      'InstanceId' => $instance_id,
      'VolumeId' => $volume_id,
      'Device' => $device_name,
    ]);

    if ($result !== NULL) {
      // Set the instance_id in the volume entity and save.
      $entity->setAttachmentInformation($instance_id);
      $entity->setState($result['State'] ?? 'in-use');
      $entity->save();
      $this->clearCacheValues();
      $this->messenger->addStatus($this->t('The Volume %volume is attaching to %instance.', [
        '%volume' => $volume_id,
        '%instance' => $instance_id,
      ]));

      $form_state->setRedirect("view.{$entity->getEntityTypeId()}.list", ['cloud_context' => $entity->getCloudContext()]);
    }
  }

  /**
   * Query DB for aws_cloud_instances that are in the same zone as the volume.
   *
   * This method respects instance visibility.
   *
   * @param string $zone
   *   The Availability Zone String.
   * @param string $module_name
   *   Module name.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return \Drupal\Core\Entity\EntityInterface[]
   *   The Instance Entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getInstances($zone, $module_name, $cloud_context = '') {

    $account = \Drupal::currentUser();
    $properties = [
      'availability_zone' => $zone,
      'cloud_context' => $cloud_context,
    ];

    // Get cloud service provider name.
    $cloud_name = self::convertUnderscoreToWhitespace($module_name);

    if (!$account->hasPermission("view any {$cloud_name} instance")) {
      $properties['uid'] = $account->id();
    }

    return $this->entityTypeManager
      ->getStorage("{$module_name}_instance")
      ->loadByProperties($properties);
  }

}

==> cloud-4.2.0.tar.gz: cloud/modules/cloud_service_providers/aws_cloud/src/Form/Ec2/VolumeAttachForm.php <==
<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\Core\Form\FormStateInterface;
use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Volume Attach form.
 */
class VolumeAttachForm extends AwsDeleteForm {

  use CloudContentEntityTrait;

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): TranslatableMarkup {
    $entity = $this->entity;

    return $this->t('Are you sure you want to attach volume: %name?', [
      '%name' => $entity->label(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText(): TranslatableMarkup {
    return $this->t('Attach');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription(): TranslatableMarkup {
    $entity = $this->entity;
    return $this->t('<h2>Volume Information:</h2><ul><li>Volume id: %id</li><li>Volume name: %name</li></ul>', [
      '%id' => $entity->getVolumeId(),
      '%name' => $entity->getName(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {
    $form = parent::buildForm($form, $form_state);

    $entity = $this->entity;
    if (!$this->updateEntityBuildForm($entity)) {
      return $this->redirectUrl;
    }

    // Get module name.
    $module_name = $this->getModuleName($entity);

    $instances = [];
    $results = $this->getInstances($this->entity->getAvailabilityZone(), $module_name, $cloud_context);

    foreach ($results ?: [] as $result) {
      /** @var \Drupal\aws_cloud\Entity\Ec2\Instance $result */
      $instances[$result->getInstanceId()] = $this->t('%name - %instance_id', [
        '%name' => $result->getName(),
        '%instance_id' => $result->getInstanceId(),
      ]);
    }
    if (count($results) > 0) {
      $form['device_name'] = [
        '#title' => $this->t('Device Name'),
        '#type' => 'textfield',
        '#description' => $this->t('The device name (for example, /dev/sdh or xvdh).'),
        '#required' => TRUE,
      ];

      $form['instance_id'] = [
        '#type' => 'select',
        '#title' => $this->t('Instance ID'),
        '#options' => $instances,
      ];
    }
    else {
      $form['message'] = [
        '#markup' => '<h1>' . $this->t('No instances available in the availability zone: %zone.  Volume cannot be attached.', ['%zone' => $this->entity->getAvailabilityZone()]) . '</h1>',
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state): array {
    $actions = parent::actions($form, $form_state);

    $entity = $this->entity;

    // Get module name.
    $module_name = $this->getModuleName($entity);

    $results = $this->getInstances($this->entity->getAvailabilityZone(), $module_name, $entity->getCloudContext());
    if (count($results) === 0) {
      unset($actions['submit']);
    }
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    /** @var \Drupal\aws_cloud\Entity\Ec2\Volume $entity */
    $entity = $this->entity;

    $instance_id = $form_state->getValue('instance_id');
    $volume_id = $entity->getVolumeId();
    $device_name = $form_state->getValue('device_name');

    $this->ec2Service->setCloudContext($this->entity->getCloudContext());
    $result = $this->ec2Service->attachVolume([
      'InstanceId' => $instance_id,
      'VolumeId' => $volume_id,
      'Device' => $device_name,
    ]);

    if ($result !== NULL) {
      $form_state->setRedirect("view.{$entity->getEntityTypeId()}.list", ['cloud_context' => $entity->getCloudContext()]);

      if (!empty($result['SendToWorker'])) {
        $this->messenger->addStatus($this->t('The Volume %volume is attaching to %instance remotely.', [
          '%volume' => $volume_id,
          '%instance' => $instance_id,
        ]));
        return;
      }

      // Set the instance_id in the volume entity and save.
      $entity->setAttachmentInformation($instance_id);
      $entity->setState($result['State'] ?? 'in-use');
      $entity->save();
      $this->clearCacheValues($entity->getCacheTags());
      $this->dispatchSubmitEvent($entity);
      $this->messenger->addStatus($this->t('The Volume %volume is attaching to %instance.', [
        '%volume' => $volume_id,
        '%instance' => $instance_id,
      ]));
    }
  }

  /**
   * Query DB for aws_cloud_instances that are in the same zone as the volume.
   *
   * This method respects instance visibility.
   *
   * @param string $zone
   *   The Availability Zone String.
   * @param string $module_name
   *   Module name.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return \Drupal\Core\Entity\EntityInterface[]
   *   The Instance Entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getInstances($zone, $module_name, $cloud_context = ''): array {

    $account = \Drupal::currentUser();
    $properties = [
      'availability_zone' => $zone,
      'cloud_context' => $cloud_context,
    ];

    // Get cloud service provider name.
    $cloud_name = self::convertUnderscoreToWhitespace($module_name);

    if (!$account->hasPermission("view any {$cloud_name} instance")) {
      $properties['uid'] = $account->id();
    }

    return $this->entityTypeManager
      ->getStorage("{$module_name}_instance")
      ->loadByProperties($properties);
  }

}
